@@ -13,13 +13,13 @@ Data::GUID - globally unique identifiers
 
 =head1 VERSION
 
-version 0.01
+version 0.02
 
- $Id: /my/cs/projects/guid/trunk/lib/Data/GUID.pm 19177 2006-02-26T04:22:56.847642Z rjbs  $
+ $Id: /my/cs/projects/guid/trunk/lib/Data/GUID.pm 19227 2006-02-28T01:06:16.070217Z rjbs  $
 
 =cut
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 =head1 SYNOPSIS
 
@@ -50,21 +50,17 @@ This method returns a new globally unique identifier.
 
 =cut
 
-sub _guid_from_uuid {
-  my ($class, $uuid) = @_;
-  bless \$uuid => $class;
-}
-
 my $_uuid_gen = Data::UUID->new;
 sub new {
   my ($class) = @_;
 
-  return $class->_guid_from_uuid($_uuid_gen->create);
+  return $class->from_data_uuid($_uuid_gen->create);
 }
 
-=head1 GUIDS FROM STRINGS
+=head1 GUIDS FROM EXISTING VALUES
 
-These method returns a new Data::GUID object for the given GUID value.
+These method returns a new Data::GUID object for the given GUID value.  In all
+cases, these methods throw an exception if given invalid input.
 
 =head2 C< from_string >
 
@@ -79,60 +75,115 @@ These method returns a new Data::GUID object for the given GUID value.
 
   my $guid = Data::GUID->from_base64("sEcGAqZLEdqGMpPr8cDgWg==");
 
+=head2 C< from_data_uuid >
+
+This method returns a new Data::GUID object if given a Data::UUID value.
+Because Data::UUID values are not blessed and because Data::UUID provides no
+validation method, this method will only throw an exception if the given data
+is of the wrong size.
+
 =cut
 
-my %from = (
-  string => 'string',
-  hex    => 'hexstring',
-  base64 => 'b64string',
+sub from_data_uuid {
+  my ($class, $value) = @_;
+
+  my $length = do { use bytes; defined $value ? length $value : 0; };
+  Carp::croak "given value is not a valid Data::UUID value" if $length != 16;
+  bless \$value => $class;
+}
+
+my $hex    = qr/[0-9A-F]/i;
+my $base64 = qr{[A-Z0-9+/=]}i;
+
+my %type = ( # uuid_method  validation_regex
+  string => [ 'string',     qr/\A$hex{8}-(?:$hex{4}-){3}$hex{12}\z/, ],
+  hex    => [ 'hexstring',  qr/\A0x$hex{32}\z/,                      ],
+  base64 => [ 'b64string',  qr/\A$base64{24}\z/,                     ],
 );
 
-do {
-  no strict 'refs';
-  while (my ($our_method, $alien_method) = each %from) {
-    my $our_from_method   = "from_$our_method";
-    my $alien_from_method = "from_$alien_method";
-    *$our_from_method = sub { 
-      my ($class, $string) = @_;
-      $class->_guid_from_uuid( $_uuid_gen->$alien_from_method($string) );
-    };
+# provided for test scripts
+sub __type_regex { shift; $type{$_[0]}[1] }
 
-    my $our_to_method   = "as_$our_method";
-    my $alien_to_method = "to_$alien_method";
-    *$our_to_method = sub { 
-      my ($self) = @_;
-      $_uuid_gen->$alien_to_method( $self->as_binary );
-    };
+sub _install_from_method {
+  my ($type, $alien_method, $regex) = @_;
+  my $alien_from_method = "from_$alien_method";
+
+  my $our_from_code = sub { 
+    my ($class, $string) = @_;
+    $string ||= ''; # to avoid (undef =~) warning
+    Carp::croak qq{"$string" is not a valid $type GUID} if $string !~ $regex;
+    $class->from_data_uuid( $_uuid_gen->$alien_from_method($string) );
+  };
+
+  Sub::Install::install_sub({ code => $our_from_code, as => "from_$type" });
+}
+
+sub _install_as_method {
+  my ($type, $alien_method) = @_;
+
+  my $alien_to_method = "to_$alien_method";
+
+  my $our_to_method = sub { 
+    my ($self) = @_;
+    $_uuid_gen->$alien_to_method( $self->as_binary );
+  };
+
+  Sub::Install::install_sub({ code => $our_to_method, as => "as_$type" });
+}
+
+do {
+  while (my ($type, $profile) = each %type) {
+    _install_from_method($type, @$profile);
+    _install_as_method  ($type, @$profile);
   }
 };
 
+sub _GUID {
+  my ($class, $value) = @_;
+  return $value if eval { $value->isa('Data::GUID') };
+
+  # The only good ref is a blessed ref, and only into our denomination!
+  return if (ref $value);
+  
+  for my $type ((keys %type), 'data_uuid') {
+    my $from = "from_$type";
+    my $guid = eval { $class->$from($value); };
+    return $guid if $guid;
+  }
+
+  return;
+}
+
 =head1 GUIDS INTO STRINGS
 
-These methods return string representations of a GUID.
+These methods return various string representations of a GUID.
 
 =head2 C< as_string >
 
+This method returns a "traditional" GUID/UUID string representation.  This is
+five hexadecimal strings, delimited by hyphens.  For example:
+
+  B0470602-A64B-11DA-8632-93EBF1C0E05A
+
 This method is also used to stringify Data::GUID objects.
 
 =head2 C< as_hex >
 
+This method returns a plain hexadecimal representation of the GUID, with a
+leading C<0x>.  For example:
+
+  0xB0470602A64B11DA863293EBF1C0E05A
+
 =head2 C< as_base64 >
 
+This method returns a base-64 string representation of the GUID.  For example:
+
+  sEcGAqZLEdqGMpPr8cDgWg==
+
 =cut
 
 =head1 OTHER METHODS
 
-=head2 C< as_binary >
-
-This method returns the packed binary representation of the GUID.
-
-=cut
-
-sub as_binary {
-  my ($self) = @_;
-  $$self;
-}
-
 =head2 C< compare_to_guid >
 
 This method compares a GUID to another GUID and returns -1, 0, or 1, as do
@@ -142,10 +193,24 @@ other comparison routines.
 
 sub compare_to_guid {
   my ($self, $other) = @_;
-  return ($self->as_binary <=> $other)
-    unless (eval { $other->isa('Data::GUID') });
 
-  $_uuid_gen->compare($self->as_binary, $other->as_binary);
+  my $other_binary
+    = eval { $other->isa('Data::GUID') } ? $other->as_binary : $other;
+
+  $_uuid_gen->compare($self->as_binary, $other_binary);
+}
+
+=head2 C< as_binary >
+
+This method returns the packed binary representation of the GUID.  At present
+this method relies on Data::GUID's underlying use of Data::UUID.  It is not
+guaranteed to continue to work the same way, or at all.  I<Caveat invocator>.
+
+=cut
+
+sub as_binary {
+  my ($self) = @_;
+  $$self;
 }
 
 use overload
@@ -186,7 +251,7 @@ This returns the base64 representation of a new GUID.
 
 { no warnings 'once'; *guid = \&new; }
 
-for my $type (keys %from) {
+for my $type (keys %type) {
   my $method = "guid_$type";
   my $as     = "as_$type";
 
@@ -197,7 +262,9 @@ for my $type (keys %from) {
   }
 }
 
-my %exports = map { $_ => 1 } ('guid', map { "guid_$_" } keys %from);
+my %exports = map { $_ => 1 } ('guid', map { "guid_$_" } keys %type);
+
+$exports{_GUID} = 1;
 
 sub import {
   my ($class, @to_export) = @_;
@@ -205,7 +272,8 @@ sub import {
   @to_export = keys %exports if grep { $_ eq ':all' } @to_export;
   
   for my $sub (@to_export) {
-    Carp::croak "$sub is not exported by Data::GUID" unless $exports{ $sub };
+    Carp::croak qq{"$sub" is not exported by the Data::GUID module}
+      unless $exports{ $sub };
     Sub::Install::install_sub({
       code => sub { $class->$sub },
       into => $into,
@@ -226,6 +294,8 @@ Ricardo SIGNES, C<< <rjbs@cpan.org> >>
 
 =item * remove dependency on wretched Data::UUID
 
+=item * make it work on 5.005
+
 =back
 
 =head1 BUGS
